from typing import Optional
from aws_cf_custom_resources.config_params import ConfigurationParameters


class GlobalConfigManager:
    __config_params: Optional[ConfigurationParameters] = None

    @staticmethod
    def set_params(config_params: ConfigurationParameters) -> None:
        if GlobalConfigManager.__config_params:
            raise ValueError('Configuration parameters already set.')

        GlobalConfigManager.__config_params = config_params

    @staticmethod
    def get_params() -> ConfigurationParameters:
        if not GlobalConfigManager.__config_params:
            raise ValueError('Configuration parameters not set!')

        return GlobalConfigManager.__config_params



import os

from setuptools import setup

setup(
    name=os.path.basename(os.path.dirname(os.path.abspath(__file__))),
    version='1.0.0',
    install_requires=[
        'boto3==1.9.215',
        'botocore==1.12.215',
        'cfnresponse==1.0.1'
    ],
)



import boto3

from typing import Any, Dict
from botocore.exceptions import ClientError

client = boto3.client('codecommit')


class Commit:
    @staticmethod
    def create(**kwargs) -> Dict[str, Any]:
        kwargs = dict(
            repositoryName=kwargs.get('repositoryName'),
            branchName=kwargs.get('branchName'),
            parentCommitId=kwargs.get('parentCommitId'),
            authorName=kwargs.get('authorName'),
            email=kwargs.get('email'),
            commitMessage=kwargs.get('commitMessage'),
            keepEmptyFolders=kwargs.get('keepEmptyFolders'),
            putFiles=kwargs.get('putFiles'),
            deleteFiles=kwargs.get('deleteFiles'),
            setFileModes=kwargs.get('setFileModes'),
        )

        try:
            kwargs = {key: value for key, value in kwargs.items() if key and value}
            return client.create_commit(**kwargs)
        except ClientError as ex:
            if ex.response['Error']['Code'] == 'ParentCommitIdRequiredException':
                latest_commit_id = client.get_branch(
                    repositoryName=kwargs.get('repositoryName'),
                    branchName=kwargs.get('branchName')
                )['branch']['commitId']

                kwargs['parentCommitId'] = latest_commit_id
                return client.create_commit(**kwargs)



import cfnresponse
import boto3
import botocore
import logging

from typing import Dict, Optional

logger = logging.getLogger()
logger.setLevel(logging.INFO)

logger.info(f'Version of boto3 lib: {boto3.__version__}.')
logger.info(f'Version of botocore lib: {botocore.__version__}.')


def __success(event, context, data):
    logger.info('SUCCESS: {}'.format(data))
    cfnresponse.send(event, context, cfnresponse.SUCCESS, data)


def __failed(event, context, data):
    logger.info('FAIL: {}'.format(data))
    cfnresponse.send(event, context, cfnresponse.FAILED, data)


def __create(user_pool_id: str, client_id: str) -> Optional[str]:
    client = boto3.client('cognito-idp')

    try:
        response = client.describe_user_pool_client(
            UserPoolId=user_pool_id,
            ClientId=client_id
        )

        logger.info(response)
    except Exception as ex:
        logger.error(repr(ex))
        return None

    try:
        return response['UserPoolClient']['ClientSecret']
    except KeyError as ex:
        logger.error(repr(ex))
        return None


def __update(user_pool_id: str, client_id: str) -> Optional:
    client = boto3.client('cognito-idp')

    try:
        response = client.describe_user_pool_client(
            UserPoolId=user_pool_id,
            ClientId=client_id
        )

        logger.info(response)
    except Exception as ex:
        logger.error(repr(ex))
        return None

    try:
        return response['UserPoolClient']['ClientSecret']
    except KeyError as ex:
        logger.error(repr(ex))
        return None


def __delete():
    # Since this function is used only to retrieve information,
    # no delete actions are required to be taken.
    pass


def __handle(event, context) -> Dict[str, Optional[str]]:
    logger.info('Got new request. Event: {}, Context: {}'.format(event, context))

    user_pool_id = event['ResourceProperties']['UserPoolId']
    client_id = event['ResourceProperties']['ClientId']

    if event['RequestType'] == 'Delete':
        __delete()
        return {'Secret': None}

    if event['RequestType'] == 'Create':
        secret = __create(user_pool_id, client_id)
        return {'Secret': secret}

    if event['RequestType'] == 'Update':
        secret = __update(user_pool_id, client_id)
        return {'Secret': secret}

    raise KeyError('Unsupported request type! Type: {}'.format(event['RequestType']))


def handler(event, context):
    try:
        data = __handle(event, context)
    except Exception as ex:
        __failed(event, context, {'Error': str(ex)})
        return

    __success(event, context, data)



import boto3

from botocore.exceptions import ClientError
from typing import Any, Dict

client = boto3.client('ecs')


class Service:
    @staticmethod
    def create(**kwargs) -> Dict[str, Any]:
        kwargs = dict(
            cluster=kwargs.get('cluster'),
            serviceName=kwargs.get('serviceName'),
            taskDefinition=kwargs.get('taskDefinition'),
            loadBalancers=kwargs.get('loadBalancers'),
            serviceRegistries=kwargs.get('serviceRegistries'),
            desiredCount=kwargs.get('desiredCount'),
            clientToken=kwargs.get('clientToken'),
            launchType=kwargs.get('launchType'),
            platformVersion=kwargs.get('platformVersion'),
            role=kwargs.get('role'),
            deploymentConfiguration=kwargs.get('deploymentConfiguration'),
            placementConstraints=kwargs.get('placementConstraints'),
            placementStrategy=kwargs.get('placementStrategy'),
            networkConfiguration=kwargs.get('networkConfiguration'),
            healthCheckGracePeriodSeconds=kwargs.get('healthCheckGracePeriodSeconds'),
            schedulingStrategy=kwargs.get('schedulingStrategy'),
            deploymentController=kwargs.get('deploymentController'),
            tags=kwargs.get('tags'),
            enableECSManagedTags=kwargs.get('enableECSManagedTags'),
            propagateTags=kwargs.get('propagateTags'),
        )

        kwargs = {key: value for key, value in kwargs.items() if key and value}
        return client.create_service(**kwargs)

    @staticmethod
    def update(**kwargs: Dict[str, Any]) -> Dict[str, Any]:
        filtered_kwargs = dict(
            cluster=kwargs.get('cluster'),
            service=kwargs.get('serviceName'),
            desiredCount=kwargs.get('desiredCount'),
            taskDefinition=kwargs.get('taskDefinition'),
            deploymentConfiguration=kwargs.get('deploymentConfiguration'),
            networkConfiguration=kwargs.get('networkConfiguration'),
            platformVersion=kwargs.get('platformVersion'),
            forceNewDeployment=kwargs.get('forceNewDeployment'),
            healthCheckGracePeriodSeconds=kwargs.get('healthCheckGracePeriodSeconds')
        )

        try:
            filtered_kwargs = {key: value for key, value in filtered_kwargs.items() if key and value}
            return client.update_service(**filtered_kwargs)
        except ClientError as ex:
            if ex.response['Error']['Code'] == 'InvalidParameterException':
                if 'use aws codedeploy' in ex.response['Error']['Message'].lower():
                    # For services using the blue/green (CODE_DEPLOY ) deployment controller,
                    # only the desired count, deployment configuration, and health check grace period
                    # can be updated using this API. If the network configuration, platform version, or task definition
                    # need to be updated, a new AWS CodeDeploy deployment should be created.
                    filtered_kwargs = dict(
                        cluster=kwargs.get('cluster'),
                        service=kwargs.get('serviceName'),
                        desiredCount=kwargs.get('desiredCount'),
                        deploymentConfiguration=kwargs.get('deploymentConfiguration'),
                        healthCheckGracePeriodSeconds=kwargs.get('healthCheckGracePeriodSeconds'),
                    )

                    filtered_kwargs = {key: value for key, value in filtered_kwargs.items() if key and value}
                    return client.update_service(**filtered_kwargs)
            elif ex.response['Error']['Code'] == 'ServiceNotActiveException':
                # We can not update ecs service if it is inactive.
                return {'Code': 'ServiceNotActiveException'}
            elif ex.response['Error']['Code'] == 'ServiceNotFoundException':
                # If for some reason service was not found - don't update and return.
                return {'Code': 'ServiceNotFoundException'}
            raise

    @staticmethod
    def delete(**kwargs: Dict[str, Any]) -> Dict[str, Any]:
        kwargs = dict(
            cluster=kwargs.get('cluster'),
            service=kwargs.get('serviceName'),
            force=True
        )

        kwargs = {key: value for key, value in kwargs.items() if key and value}
        return client.delete_service(**kwargs)



from troposphere.cloudformation import AWSCustomObject


class CustomCognitoDomain(AWSCustomObject):
    """
    Custom cloud-formation resource used to create a domain name for AWS Cognito user pool.
    """
    resource_type = "Custom::CognitoDomain"

    props = {
        'ServiceToken': (str, True),
        'UserPoolId': (str, True),
        'DomainName': (str, True),
    }



from troposphere import AWSObject


class CustomDeploymentGroup(AWSObject):
    """
    Custom cloud-formation resource used to create a deployment group for AWS CodeDeploy.
    """
    resource_type = "Custom::DeploymentGroup"

    props = {
        'ServiceToken': (str, True),
        'ApplicationName': (str, True),
        'DeploymentGroupName': (str, True),
        'DeploymentConfigName': (str, False),
        'Ec2TagFilters': ([dict], False),
        'OnPremisesInstanceTagFilters': ([dict], False),
        'AutoScalingGroups': ([str], False),
        'ServiceRoleArn': (str, True),
        'TriggerConfigurations': ([dict], False),
        'AlarmConfiguration': (dict, False),
        'AutoRollbackConfiguration': (dict, False),
        'DeploymentStyle': (dict, False),
        'BlueGreenDeploymentConfiguration': (dict, False),
        'LoadBalancerInfo': (dict, False),
        'Ec2TagSet': (dict, False),
        'EcsServices': ([dict], False),
        'OnPremisesTagSet': (dict, False),
        'Tags': ([dict], False)
    }



class ConfigurationParameters:
    __CF_CUSTOM_BUILDS_BUCKET = 'aws.infrastructure.sdk.cf.custom.resources.{}'

    def __init__(
            self,
            custom_resources_bucket_suffix: str
    ) -> None:
        assert custom_resources_bucket_suffix
        self.__custom_resources_bucket_suffix: str = custom_resources_bucket_suffix

    @property
    def custom_resources_bucket(self) -> str:
        return self.__CF_CUSTOM_BUILDS_BUCKET.format(self.__custom_resources_bucket_suffix)



import json
import cfnresponse
import boto3
import botocore
import logging

from botocore.exceptions import ClientError

try:
    from aws_cf_custom_resources.ecs_service.package.fixer import Fixer
    from aws_cf_custom_resources.ecs_service.package.service import Service
except ImportError:
    # Lambda specific import.
    # noinspection PyUnresolvedReferences
    from fixer import Fixer
    # noinspection PyUnresolvedReferences
    from service import Service

logger = logging.getLogger()
logger.setLevel(logging.INFO)

logger.info(f'Version of boto3 lib: {boto3.__version__}.')
logger.info(f'Version of botocore lib: {botocore.__version__}.')


def __success(event, context, data):
    logger.info('SUCCESS: {}'.format(data))
    cfnresponse.send(event, context, cfnresponse.SUCCESS, data)


def __failed(event, context, data):
    logger.info('FAIL: {}'.format(data))
    cfnresponse.send(event, context, cfnresponse.FAILED, data)


def __create(**kwargs):
    response = Service.create(**kwargs)
    logger.info(json.dumps(response, default=lambda o: '<not serializable>'))


def __update(**kwargs):
    response = Service.update(**kwargs)
    logger.info(json.dumps(response, default=lambda o: '<not serializable>'))


def __delete(**kwargs):
    response = Service.delete(**kwargs)
    logger.info(json.dumps(response, default=lambda o: '<not serializable>'))


def __handle(event, context):
    logger.info('Got new request. Event: {}, Context: {}'.format(event, context))

    kwargs = event['ResourceProperties']
    kwargs = Fixer(kwargs).fix().get()

    if event['RequestType'] == 'Delete':
        return __delete(**kwargs)

    if event['RequestType'] == 'Create':
        return __create(**kwargs)

    if event['RequestType'] == 'Update':
        return __update(**kwargs)

    raise KeyError('Unsupported request type! Type: {}'.format(event['RequestType']))


def handler(event, context):
    try:
        __handle(event, context)
    except ClientError as ex:
        return __failed(event, context, {'Error': str(ex.response)})
    except Exception as ex:
        return __failed(event, context, {'Error': str(ex)})

    __success(event, context, {'Success': True})



import cfnresponse
import boto3
import botocore
import json
import logging

logger = logging.getLogger()
logger.setLevel(logging.INFO)

logger.info(f'Version of boto3 lib: {boto3.__version__}.')
logger.info(f'Version of botocore lib: {botocore.__version__}.')

client = boto3.client('cognito-idp')


def __success(event, context, data):
    logger.info('SUCCESS: {}'.format(data))
    cfnresponse.send(event, context, cfnresponse.SUCCESS, data)


def __failed(event, context, data):
    logger.info('FAIL: {}'.format(data))
    cfnresponse.send(event, context, cfnresponse.FAILED, data)


def __create(user_pool_id: str, facebook_client_id: str, facebook_client_secret: str):
    response = client.create_identity_provider(
        UserPoolId=user_pool_id,
        ProviderName='Facebook',
        ProviderType='Facebook',
        ProviderDetails={
            'client_id': facebook_client_id,
            'client_secret': facebook_client_secret,
            "authorize_scopes": "email"
        },
        AttributeMapping={
            'username': 'id',
            'email': 'email'
        }
    )

    logger.info(
        f"Response from creating user pool identity provider: "
        f"{json.dumps(response, default=lambda o: '<not serializable>')}"
    )


def __update(user_pool_id: str, facebook_client_id: str, facebook_client_secret: str):
    response = client.update_identity_provider(
        UserPoolId=user_pool_id,
        ProviderName='Facebook',
        ProviderDetails={
            'client_id': facebook_client_id,
            'client_secret': facebook_client_secret,
            "authorize_scopes": "email"
        },
        AttributeMapping={
            'username': 'id',
            'email': 'email'
        }
    )

    logger.info(
        f"Response from updating user pool identity provider: "
        f"{json.dumps(response, default=lambda o: '<not serializable>')}"
    )


def __delete(user_pool_id: str):
    try:
        response = client.delete_identity_provider(
            UserPoolId=user_pool_id,
            ProviderName='Facebook'
        )

        logger.info(
            f"Response from deleting user pool identity provider: "
            f"{json.dumps(response, default=lambda o: '<not serializable>')}"
        )
    except Exception as ex:
        logger.error(repr(ex))


def __handle(event, context):
    logger.info('Got new request. Event: {}, Context: {}'.format(event, context))

    user_pool_id = event['ResourceProperties']['UserPoolId']
    facebook_client_id = event['ResourceProperties']['FacebookClientId']
    facebook_client_secret = event['ResourceProperties']['FacebookClientSecret']

    if event['RequestType'] == 'Delete':
        __delete(user_pool_id)
        return

    if event['RequestType'] == 'Create':
        __create(user_pool_id, facebook_client_id, facebook_client_secret)
        return

    if event['RequestType'] == 'Update':
        __update(user_pool_id, facebook_client_id, facebook_client_secret)
        return

    raise KeyError('Unsupported request type! Type: {}'.format(event['RequestType']))


def handler(event, context):
    try:
        __handle(event, context)
    except Exception as ex:
        __failed(event, context, {'Error': str(ex)})
        return

    __success(event, context, {'Success': True})



from troposphere.cloudformation import AWSCustomObject


class CustomCognitoFacebook(AWSCustomObject):
    """
    Custom cloud-formation resource used to create a facebook identity provider for AWS Cognito user pool.
    """
    resource_type = "Custom::CognitoFacebook"

    props = {
        'ServiceToken': (str, True),
        'UserPoolId': (str, True),
        'FacebookClientId': (str, True),
        'FacebookClientSecret': (str, True),
    }



from troposphere import AWSObject
from troposphere.validators import boolean


class CustomGitCommit(AWSObject):
    resource_type = "Custom::GitCommit"

    props = {
        'ServiceToken': (str, True),
        'RepositoryName': (str, True),
        'BranchName': (str, True),
        'ParentCommitId': (str, False),
        'AuthorName': (str, False),
        'Email': (str, False),
        'CommitMessage': (str, False),
        'KeepEmptyFolders': (bool, False),
        'PutFiles': ([dict], False),
        'DeleteFiles': ([dict], False),
        'SetFileModes': ([dict], False),
    }



from setuptools import setup, find_packages

setup(
    name='aws_cf_custom_resources',
    version='1.0.1',
    license='GNU GENERAL PUBLIC LICENSE Version 3',
    packages=find_packages(),
    description='Container library which contains various custom resources for AWS CloudFormation.',
    include_package_data=True,
    install_requires=[
        'boto3',
        'botocore',
        'troposphere',
        'cfnresponse',
        'aws-lambda'
    ],
    author='Laimonas Sutkus',
    author_email='laimonas.sutkus@gmail.com',
    keywords='AWS SDK CloudFormation Infrastructure Cloud CustomResource',
    url='https://github.com/laimonassutkus/AwsCfCustomResources',
    classifiers=[
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Operating System :: OS Independent',
    ],
)
